@@ -52,7 +52,7 @@
 #include <iostream>
 #include <exception>
 
-int main (void) {
+int main () {
     // create a new XML document and set the root node
     xml::document xmldoc(xml::node("abook"));
     xml::node &root = xmldoc.get_root_node();
