@@ -62,7 +62,7 @@ ait_impl::ait_impl(xmlNodePtr node, xmlAttrPtr prop)
 
 
 ait_impl::ait_impl(const char *name, const char *value, bool)
-    : xmlnode_(0), xmlattr_(0), fake_(true)
+    : xmlnode_(nullptr), xmlattr_(nullptr), fake_(true)
 {
     // in this constructor and in the functions to follow, the last
     // parameter, the bool, is only used to create a unique signature
@@ -132,26 +132,26 @@ ait_impl ait_impl::operator++(int)
 // ------------------------------------------------------------------------
 
 attributes::iterator::iterator()
+    : pimpl_{new ait_impl(nullptr, nullptr)}
 {
-    pimpl_ = new ait_impl(0, 0);
 }
 
 
 attributes::iterator::iterator(void *node, void *prop)
+    : pimpl_{new ait_impl(static_cast<xmlNodePtr>(node), static_cast<xmlAttrPtr>(prop))}
 {
-    pimpl_ = new ait_impl(static_cast<xmlNodePtr>(node), static_cast<xmlAttrPtr>(prop));
 }
 
 
 attributes::iterator::iterator(const char *name, const char *value, bool)
+    : pimpl_{new ait_impl(name, value, true)}
 {
-    pimpl_ = new ait_impl(name, value, true);
 }
 
 
 attributes::iterator::iterator (const iterator &other)
+    : pimpl_{new ait_impl(*other.pimpl_)}
 {
-    pimpl_ = new ait_impl(*other.pimpl_);
 }
 
 
@@ -169,10 +169,7 @@ void attributes::iterator::swap(iterator& other)
 }
 
 
-attributes::iterator::~iterator()
-{
-    delete pimpl_;
-}
+attributes::iterator::~iterator() = default;
 
 
 void* attributes::iterator::get_raw_attr()
@@ -213,32 +210,32 @@ attributes::iterator attributes::iterator::operator++(int)
 // ------------------------------------------------------------------------
 
 attributes::const_iterator::const_iterator()
+    : pimpl_{new ait_impl(nullptr, nullptr)}
 {
-    pimpl_ = new ait_impl(0, 0);
 }
 
 
 attributes::const_iterator::const_iterator(void *node, void *prop)
+    : pimpl_{new ait_impl(static_cast<xmlNodePtr>(node), static_cast<xmlAttrPtr>(prop))}
 {
-    pimpl_ = new ait_impl(static_cast<xmlNodePtr>(node), static_cast<xmlAttrPtr>(prop));
 }
 
 
 attributes::const_iterator::const_iterator(const char *name, const char *value, bool)
+    : pimpl_{new ait_impl(name, value, true)}
 {
-    pimpl_ = new ait_impl(name, value, true);
 }
 
 
 attributes::const_iterator::const_iterator(const const_iterator& other)
+    : pimpl_{new ait_impl(*other.pimpl_)}
 {
-    pimpl_ = new ait_impl(*other.pimpl_);
 }
 
 
 attributes::const_iterator::const_iterator(const iterator& other)
+    : pimpl_{new ait_impl(*other.pimpl_)}
 {
-    pimpl_ = new ait_impl(*other.pimpl_);
 }
 
 
@@ -256,10 +253,7 @@ void attributes::const_iterator::swap(const_iterator& other)
 }
 
 
-attributes::const_iterator::~const_iterator()
-{
-    delete pimpl_;
-}
+attributes::const_iterator::~const_iterator() = default;
 
 
 void* attributes::const_iterator::get_raw_attr()
@@ -299,28 +293,6 @@ attributes::const_iterator attributes::const_iterator::operator++(int)
 // xml::attributes::attr
 // ------------------------------------------------------------------------
 
-attributes::attr::attr() : node_(0), prop_(0)
-{
-}
-
-
-attributes::attr::attr(const attr& other)
-    : node_(other.node_),
-      prop_(other.prop_),
-      name_(other.name_),
-      value_(other.value_)
-{
-}
-
-
-attributes::attr& attributes::attr::operator=(const attr& other)
-{
-    attr tmp(other);
-    swap(tmp);
-    return *this;
-}
-
-
 void attributes::attr::swap(attr& other)
 {
     std::swap(node_, other.node_);
@@ -341,8 +313,8 @@ void attributes::attr::set_data(void *node, void *prop)
 
 void attributes::attr::set_data(const char *name, const char *value, bool)
 {
-    node_ = 0;
-    prop_ = 0;
+    node_ = nullptr;
+    prop_ = nullptr;
     name_ = name;
     value_ = value;
 }
@@ -369,7 +341,7 @@ const char* attributes::attr::get_value() const
         throw xml::exception("access to invalid attributes::attr object!");
 
     xmlChar *tmpstr = xmlNodeListGetString(reinterpret_cast<xmlNodePtr>(node_)->doc, reinterpret_cast<xmlAttrPtr>(prop_)->children, 1);
-    if (tmpstr == 0)
+    if (tmpstr == nullptr)
         return "";
 
     xmlchar_helper helper(tmpstr);
@@ -415,31 +387,31 @@ xmlAttrPtr find_prop(xmlNodePtr xmlnode, const char *name)
             return prop;
     }
 
-    return 0;
+    return nullptr;
 }
 
 
 xmlAttributePtr find_default_prop(xmlNodePtr xmlnode, const char *name)
 {
-    if (xmlnode->doc != 0)
+    if (xmlnode->doc != nullptr)
     {
-        xmlAttributePtr dtd_attr=0;
+        xmlAttributePtr dtd_attr=nullptr;
 
-        if (xmlnode->doc->intSubset != 0)
+        if (xmlnode->doc->intSubset != nullptr)
         {
             dtd_attr = xmlGetDtdAttrDesc(xmlnode->doc->intSubset, xmlnode->name, reinterpret_cast<const xmlChar*>(name));
         }
 
-        if (dtd_attr == 0 && xmlnode->doc->extSubset != 0)
+        if (dtd_attr == nullptr && xmlnode->doc->extSubset != nullptr)
         {
             dtd_attr = xmlGetDtdAttrDesc(xmlnode->doc->extSubset, xmlnode->name, reinterpret_cast<const xmlChar*>(name));
         }
 
-        if (dtd_attr != 0 && dtd_attr->defaultValue != 0)
+        if (dtd_attr != nullptr && dtd_attr->defaultValue != nullptr)
             return dtd_attr;
     }
 
-    return 0;
+    return nullptr;
 }
 
 bool operator==(const ait_impl& lhs, const ait_impl& rhs)
