@@ -36,6 +36,9 @@
 #include "utility.h"
 #include "errors_impl.h"
 
+#include <cstring>
+#include <sstream>
+
 namespace xml
 {
 
@@ -43,31 +46,19 @@ error_handler_ignore_errors              ignore_errors;
 error_handler_throw_on_error             throw_on_error;
 error_handler_throw_on_error_or_warning  throw_on_error_or_warning;
 
-// ------------------------------------------------------------------------
-// xml::exception
-// ------------------------------------------------------------------------
-
-exception::exception(const std::string& what_) : std::runtime_error(what_)
-{
-}
-
-exception::exception(const error_messages& what_) : std::runtime_error(what_.print())
-{
-}
-
 // ------------------------------------------------------------------------
 // xml::error_messages
 // ------------------------------------------------------------------------
 
 void error_messages::on_error(const std::string& msg)
 {
-    messages_.push_back(error_message(msg, error_message::type_error));
+    messages_.emplace_back(msg, error_message::type_error);
     has_errors_ = true;
 }
 
 void error_messages::on_warning(const std::string& msg)
 {
-    messages_.push_back(error_message(msg, error_message::type_warning));
+    messages_.emplace_back(msg, error_message::type_warning);
     has_warnings_ = true;
 }
 
@@ -75,13 +66,11 @@ std::string error_messages::print() const
 {
     std::string buffer;
 
-    messages_type::const_iterator begin(messages_.begin());
-    messages_type::const_iterator end(messages_.end());
-    for (messages_type::const_iterator k = begin; k != end; ++k)
+    for (const auto& msg : messages_)
     {
-        if (k != begin)
+        if (!buffer.empty())
             buffer += "\n";
-        buffer += format_for_print(*k);
+        buffer += format_for_print(msg);
     }
 
     return buffer;
@@ -117,11 +106,136 @@ namespace impl
 // desirable). So we collect the errors and "replay" them after returning from
 // C code.
 
+std::string build_message(const std::string& what, const xmlError& error)
+{
+    std::ostringstream oss;
+
+    // We currently don't decode domain and code to their symbolic
+    // representation as it doesn't seem to be worth it in practice, the error
+    // message is usually clear enough, while these numbers can be used for
+    // automatic classification of messages.
+    oss << "XML " << what << " " << error.domain << "." << error.code << ": "
+        << error.message;
+
+    if (error.file)
+    {
+        oss << " at " << error.file;
+        if (error.line)
+        {
+            oss << ":" << error.line;
+
+            // Column information, if available, is passed in the second int
+            // field (first one is used with the first string field, see below).
+            if (error.int2)
+            {
+                oss << "," << error.int2;
+            }
+        }
+    }
+
+    if (error.str1)
+    {
+        oss << " while processing \"" << error.str1 << "\"";
+        if (error.int1)
+        {
+            oss << " at position " << error.int1;
+        }
+    }
+
+    return oss.str();
+}
+
+bool treat_warning_as_error(const xmlError& error)
+{
+    // Currently we only handle warnings with this error code as errors.
+    if (error.code != XML_IO_LOAD_ERROR)
+        return false;
+
+    // But even warnings using this error code are not always errors, as we
+    // also get them when a DTD referenced in the file being parsed can't be
+    // opened -- which is really just a warning.
+    //
+    // So check that the warning is really about the file itself by checking
+    // the error message, which always references the file in this case. Be
+    // careful to not suppose that either file or str1 fields are set, as
+    // "file" field is NULL and the XML file path is in "str1" for the warnings
+    // concerning the file itself while warnings about loading DTD contain DTD
+    // itself in "str1" and the XML file being processed is in "file".
+    std::string load_error = "failed to load external entity \"";
+    load_error += error.file ? error.file : error.str1 ? error.str1 : "";
+    load_error += "\"";
+
+    if (error.message != load_error)
+        return false;
+
+    // We really failed to open the file being processed, this is definitely an
+    // error.
+    return true;
+}
+
+#if LIBXML_VERSION >= 21200
+extern "C" void cb_messages_structured_error(void *out, const xmlError *error)
+#else
+extern "C" void cb_messages_structured_error(void *out, xmlErrorPtr error)
+#endif
+{
+    try
+    {
+        // This is not supposed to happen, but don't crash if it ever does.
+        if (!error)
+            return;
+
+        // Chop the trailing new line, if any, it's worse than useless when not
+        // sending the output directly to the terminal.
+        if (error->message)
+        {
+            const std::size_t len = std::strlen(error->message);
+            if (len && error->message[len - 1] == '\n')
+                error->message[len - 1] = '\0';
+        }
+
+        auto* const messages = static_cast<error_messages*>(out);
+
+        switch (error->level)
+        {
+            case XML_ERR_NONE:
+                break;
+
+            case XML_ERR_WARNING:
+                // Some libxml warnings are pretty fatal errors, e.g. failing
+                // to open the input file is reported as a warning with code
+                // XML_IO_LOAD_ERROR, so treat them as such.
+                if (!treat_warning_as_error(*error))
+                {
+                    messages->on_warning(build_message("warning", *error));
+                    return;
+                }
+                // fallthrough
+
+            case XML_ERR_ERROR:
+                messages->on_error(build_message("error", *error));
+                return;
+
+            case XML_ERR_FATAL:
+                messages->on_error(build_message("fatal error", *error));
+                return;
+        }
+
+        // This is not supposed to happen neither, but at least warn about it
+        // if it does, in case there is anything useful in the message.
+        std::ostringstream oss;
+        oss << "message of unknown level " << error->level;
+
+        messages->on_warning(build_message(oss.str(), *error));
+    }
+    catch (...) {}
+}
+
 extern "C" void cb_messages_warning_v(void *out, const char *message, va_list ap)
 {
     try
     {
-        error_messages *err = static_cast<error_messages*>(out);
+        auto *err = static_cast<error_messages*>(out);
 
         std::string text;
         printf2string(text, message, ap);
@@ -134,7 +248,7 @@ extern "C" void cb_messages_error_v(void *out, const char *message, va_list ap)
 {
     try
     {
-        error_messages *err = static_cast<error_messages*>(out);
+        auto *err = static_cast<error_messages*>(out);
 
         std::string text;
         printf2string(text, message, ap);
@@ -156,19 +270,15 @@ extern "C" void cb_messages_error(void *out, const char *message, ...)
 
 void errors_collector::replay(error_handler& dest)
 {
-    const messages_type& msg = messages();
-
-    for ( messages_type::const_iterator i = msg.begin();
-          i != msg.end();
-          ++i )
+    for (const auto& msg : messages())
     {
-        switch ( i->type() )
+        switch (msg.type())
         {
             case error_message::type_error:
-                dest.on_error(i->message());
+                dest.on_error(msg.message());
                 break;
             case error_message::type_warning:
-                dest.on_warning(i->message());
+                dest.on_warning(msg.message());
                 break;
         }
     }
@@ -198,15 +308,26 @@ std::string errors_collector::format_for_print(const error_message& msg) const
 // ----------------------------------------------------------------------------
 
 global_errors_installer::global_errors_installer(error_messages& on_error) :
-    xml_error_orig_(xmlGenericError),
-    xml_error_context_orig_(xmlGenericErrorContext)
+    xml_generic_error_orig_(xmlGenericError),
+    xml_generic_error_context_orig_(xmlGenericErrorContext),
+    xml_structured_error_orig_(xmlStructuredError),
+    xml_structured_error_context_orig_(xmlStructuredErrorContext)
 {
+    // We need to set both structured and generic error callbacks: structured
+    // one because it gives us more information and does it in atomic way,
+    // instead of splitting a single error message in multiple parts and
+    // overrides generic for normal XML errors, but also generic one to catch
+    // errors from e.g. XML saving code which doesn't use structured error
+    // callback at all and would just silently (!) lose data if any errors
+    // happen during encoding without this.
+    xmlSetStructuredErrorFunc(&on_error, cb_messages_structured_error);
     xmlSetGenericErrorFunc(&on_error, cb_messages_error);
 }
 
 global_errors_installer::~global_errors_installer()
 {
-    xmlSetGenericErrorFunc(xml_error_context_orig_, xml_error_orig_);
+    xmlSetGenericErrorFunc(xml_generic_error_context_orig_, xml_generic_error_orig_);
+    xmlSetStructuredErrorFunc(xml_structured_error_context_orig_, xml_structured_error_orig_);
 }
 
 } // namespace impl
