@@ -36,7 +36,6 @@
 #include "xmlwrapp/nodes_view.h"
 #include "xmlwrapp/attributes.h"
 #include "xmlwrapp/errors.h"
-#include "cpp11.h"
 #include "utility.h"
 #include "ait_impl.h"
 #include "node_manip.h"
@@ -72,23 +71,23 @@ namespace impl
 
 struct node_impl
 {
-    node_impl() : xmlnode_(0), owner_(true), attrs_(0) {}
-    ~node_impl() { release(); }
+    node_impl() : attrs_(0) {}
+    ~node_impl() { free_node_if_owned(); }
 
-    void release()
+    void free_node_if_owned()
     {
         if (xmlnode_ && owner_)
             xmlFreeNode(xmlnode_);
     }
 
-    xmlNodePtr xmlnode_;
-    bool owner_;
+    xmlNodePtr xmlnode_{nullptr};
+    bool owner_{true};
     attributes attrs_;
     std::string tmp_string;
 };
 
 
-struct node_cmp : public std::binary_function<xmlNodePtr, xmlNodePtr, bool>
+struct node_cmp
 {
     node_cmp (cbfo_node_compare &cb) : cb_(cb) {}
 
@@ -119,9 +118,9 @@ class node2doc
 {
 public:
     node2doc(xmlNodePtr xmlnode_)
-        : xmlnode__(xmlnode_), prev_(0), next_(0)
+        : xmlnode__(xmlnode_)
     {
-        xmldoc_ = xmlNewDoc(0);
+        xmldoc_ = xmlNewDoc(nullptr);
         if (!xmldoc_)
             throw std::bad_alloc();
 
@@ -134,8 +133,8 @@ public:
 
     ~node2doc()
     {
-        xmldoc_->children   = 0;
-        xmldoc_->last   = 0;
+        xmldoc_->children   = nullptr;
+        xmldoc_->last   = nullptr;
 
         xmlFreeDoc(xmldoc_);
 
@@ -148,27 +147,27 @@ public:
 private:
     xmlDocPtr xmldoc_;
     xmlNodePtr xmlnode__;
-    xmlNodePtr prev_;
-    xmlNodePtr next_;
+    xmlNodePtr prev_{nullptr};
+    xmlNodePtr next_{nullptr};
 };
 
 
 // sort compare function to sort based on attribute
-struct compare_attr : public std::binary_function<xmlNodePtr, xmlNodePtr, bool>
+struct compare_attr
 {
     compare_attr(const char *attr_name) : name_(attr_name) {}
 
     bool operator()(xmlNodePtr lhs, xmlNodePtr rhs)
     {
         xmlAttrPtr attr_l, attr_r;
-        xmlAttributePtr dtd_l(0), dtd_r(0);
+        xmlAttributePtr dtd_l(nullptr), dtd_r(nullptr);
 
         attr_l = find_prop(lhs, name_);
-        if (attr_l == 0 && (dtd_l = find_default_prop(lhs, name_)) == 0)
+        if (attr_l == nullptr && (dtd_l = find_default_prop(lhs, name_)) == nullptr)
             return true;
 
         attr_r = find_prop(rhs, name_);
-        if (attr_r == 0 && (dtd_r = find_default_prop(rhs, name_)) == 0)
+        if (attr_r == nullptr && (dtd_r = find_default_prop(rhs, name_)) == nullptr)
             return false;
 
         xmlChar *value_l, *value_r;
@@ -198,7 +197,7 @@ struct compare_attr : public std::binary_function<xmlNodePtr, xmlNodePtr, bool>
 
 
 // add a node as a child
-struct insert_node : public std::unary_function<xmlNodePtr, void>
+struct insert_node
 {
     insert_node(xmlNodePtr parent) : parent_(parent) {}
     void operator()(xmlNodePtr child) { xmlAddChild(parent_, child); }
@@ -210,7 +209,7 @@ struct insert_node : public std::unary_function<xmlNodePtr, void>
 // an element node finder
 xmlNodePtr find_element(const char *name, xmlNodePtr first)
 {
-    while (first != 0)
+    while (first != nullptr)
     {
         if (first->type == XML_ELEMENT_NODE && xmlStrcmp(first->name, reinterpret_cast<const xmlChar*>(name)) == 0)
         {
@@ -219,27 +218,27 @@ xmlNodePtr find_element(const char *name, xmlNodePtr first)
         first = first->next;
     }
 
-    return 0;
+    return nullptr;
 }
 
 
 xmlNodePtr find_element(xmlNodePtr first)
 {
-    while (first != 0)
+    while (first != nullptr)
     {
         if (first->type == XML_ELEMENT_NODE)
             return first;
         first = first->next;
     }
 
-    return 0;
+    return nullptr;
 }
 
 
 class next_element_functor : public iter_advance_functor
 {
 public:
-    virtual xmlNodePtr operator()(xmlNodePtr node) const
+    xmlNodePtr operator()(xmlNodePtr node) const override
         { return find_element(node->next); }
 };
 
@@ -248,7 +247,7 @@ class next_named_element_functor : public iter_advance_functor
 {
 public:
     next_named_element_functor(const char *name) : name_(name) {}
-    virtual xmlNodePtr operator()(xmlNodePtr node) const
+    xmlNodePtr operator()(xmlNodePtr node) const override
         { return find_element(name_.c_str(), node->next); }
 private:
     std::string name_;
@@ -262,40 +261,33 @@ private:
 // ------------------------------------------------------------------------
 
 node::node(int)
+    : pimpl_{new node_impl}
 {
-    pimpl_ = new node_impl;
 }
 
 
 node::node()
+    : pimpl_{new node_impl}
 {
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
-    pimpl_->xmlnode_ = xmlNewNode(0, reinterpret_cast<const xmlChar*>("blank"));
+    pimpl_->xmlnode_ = xmlNewNode(nullptr, reinterpret_cast<const xmlChar*>("blank"));
     if (!pimpl_->xmlnode_)
         throw std::bad_alloc();
-
-    ap.release();
 }
 
 
 node::node (const char *name)
+    : pimpl_{new node_impl}
 {
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
-    pimpl_->xmlnode_ = xmlNewNode(0, reinterpret_cast<const xmlChar*>(name));
+    pimpl_->xmlnode_ = xmlNewNode(nullptr, reinterpret_cast<const xmlChar*>(name));
     if (!pimpl_->xmlnode_)
         throw std::bad_alloc();
-
-    ap.release();
 }
 
 
 node::node (const char *name, const char *content)
+    : pimpl_{new node_impl}
 {
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
-    pimpl_->xmlnode_ = xmlNewNode(0, reinterpret_cast<const xmlChar*>(name));
+    pimpl_->xmlnode_ = xmlNewNode(nullptr, reinterpret_cast<const xmlChar*>(name));
     if (!pimpl_->xmlnode_)
         throw std::bad_alloc();
 
@@ -311,74 +303,49 @@ node::node (const char *name, const char *content)
             throw std::bad_alloc();
         }
     }
-
-    ap.release();
 }
 
 
 node::node(cdata cdata_info)
+    : pimpl_{new node_impl}
 {
     const int len = xml::impl::checked_int_cast(std::strlen(cdata_info.t));
 
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
-    if ( (pimpl_->xmlnode_ = xmlNewCDataBlock(0, reinterpret_cast<const xmlChar*>(cdata_info.t), len)) == 0)
-    {
+    if ( (pimpl_->xmlnode_ = xmlNewCDataBlock(nullptr, reinterpret_cast<const xmlChar*>(cdata_info.t), len)) == nullptr)
         throw std::bad_alloc();
-    }
-
-    ap.release();
 }
 
 
 node::node(comment comment_info)
+    : pimpl_{new node_impl}
 {
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
-    if ( (pimpl_->xmlnode_ =  xmlNewComment(reinterpret_cast<const xmlChar*>(comment_info.t))) == 0)
-    {
+    if ( (pimpl_->xmlnode_ =  xmlNewComment(reinterpret_cast<const xmlChar*>(comment_info.t))) == nullptr)
         throw std::bad_alloc();
-    }
-
-    ap.release();
 }
 
 
 node::node(pi pi_info)
+    : pimpl_{new node_impl}
 {
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
-    if ( (pimpl_->xmlnode_ = xmlNewPI(reinterpret_cast<const xmlChar*>(pi_info.n), reinterpret_cast<const xmlChar*>(pi_info.c))) == 0)
-    {
+    if ( (pimpl_->xmlnode_ = xmlNewPI(reinterpret_cast<const xmlChar*>(pi_info.n), reinterpret_cast<const xmlChar*>(pi_info.c))) == nullptr)
         throw std::bad_alloc();
-    }
-
-    ap.release();
 }
 
 
 node::node(text text_info)
+    : pimpl_{new node_impl}
 {
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
-    if ( (pimpl_->xmlnode_ =  xmlNewText(reinterpret_cast<const xmlChar*>(text_info.t))) == 0)
-    {
+    if ( (pimpl_->xmlnode_ =  xmlNewText(reinterpret_cast<const xmlChar*>(text_info.t))) == nullptr)
         throw std::bad_alloc();
-    }
-
-    ap.release();
 }
 
 
 node::node(const node& other)
+    : pimpl_{new node_impl}
 {
-    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
-
     pimpl_->xmlnode_ = xmlCopyNode(other.pimpl_->xmlnode_, 1);
     if (!pimpl_->xmlnode_)
         throw std::bad_alloc();
-
-    ap.release();
 }
 
 
@@ -437,7 +404,7 @@ void node::move_under(node& new_parent)
     // Update the pointers inside this node itself.
     this_node->parent = new_parent_node;
     this_node->prev = new_parent_node->last;
-    this_node->next = NULL;
+    this_node->next = nullptr;
 
     // And update the new parent too.
     if (this_node->prev)
@@ -448,15 +415,12 @@ void node::move_under(node& new_parent)
 }
 
 
-node::~node()
-{
-    delete pimpl_;
-}
+node::~node() = default;
 
 
 void node::set_node_data(void *data)
 {
-    pimpl_->release();
+    pimpl_->free_node_if_owned();
     pimpl_->xmlnode_ = static_cast<xmlNodePtr>(data);
     pimpl_->owner_ = false;
 }
@@ -507,7 +471,7 @@ const char* node::get_content() const
 {
     xmlchar_helper content(xmlNodeGetContent(pimpl_->xmlnode_));
     if (!content.get())
-        return NULL;
+        return nullptr;
 
     pimpl_->tmp_string = content.get();
     return pimpl_->tmp_string.c_str();
@@ -583,18 +547,18 @@ const char *node::get_namespace() const
 {
     return pimpl_->xmlnode_->ns
         ? reinterpret_cast<const char*>(pimpl_->xmlnode_->ns->href)
-        : NULL;
+        : nullptr;
 }
 
 
 void node::set_namespace(const std::string& href)
 {
-    const xmlChar *xmlHref = reinterpret_cast<const xmlChar*>(href.c_str());
+    const auto *xmlHref = reinterpret_cast<const xmlChar*>(href.c_str());
 
     if (pimpl_->xmlnode_->type != XML_ELEMENT_NODE)
         throw xml::exception("set_namespace called on non-element node");
 
-    xmlNsPtr ns = xmlNewNs(pimpl_->xmlnode_, xmlHref, NULL);
+    xmlNsPtr ns = xmlNewNs(pimpl_->xmlnode_, xmlHref, nullptr);
 
     if ( !ns )
     {
@@ -602,7 +566,7 @@ void node::set_namespace(const std::string& href)
         // we must change its URI.
         for ( ns = pimpl_->xmlnode_->nsDef; ns; ns = ns->next )
         {
-            if ( ns->prefix == NULL )
+            if ( ns->prefix == nullptr )
             {
                 xmlFree(const_cast<xmlChar*>(ns->href));
                 ns->href = xmlStrdup(xmlHref);
@@ -613,7 +577,16 @@ void node::set_namespace(const std::string& href)
             throw xml::exception("set_namespace failed to create namespace object");
     }
 
-    xmlSetNs(pimpl_->xmlnode_, ns);
+    // If we already have a namespace, all our children without their own
+    // namespaces should already use the same pointer, so their namespaces will
+    // be updated when our namespace is changed. However if we don't have any
+    // namespace yet, children namespaces will remain unset, which would break
+    // the expected namespace inheritance and so we need to set them explicitly
+    // to avoid this.
+    if ( pimpl_->xmlnode_->ns )
+        xmlSetNs(pimpl_->xmlnode_, ns);
+    else
+        node_set_ns_recursively(pimpl_->xmlnode_, ns);
 }
 
 
@@ -625,7 +598,7 @@ bool node::is_text() const
 
 void node::push_back (const node &child)
 {
-    xml::impl::node_insert(pimpl_->xmlnode_, 0, child.pimpl_->xmlnode_);
+    xml::impl::node_insert(pimpl_->xmlnode_, nullptr, child.pimpl_->xmlnode_);
 }
 
 
@@ -638,7 +611,7 @@ node::size_type node::size() const
 
 bool node::empty() const
 {
-    return pimpl_->xmlnode_->children == 0;
+    return pimpl_->xmlnode_->children == nullptr;
 }
 
 
@@ -702,8 +675,8 @@ node::const_iterator node::find(const char *name) const
 
 node::iterator node::find(const char *name, const iterator& start)
 {
-    xmlNodePtr n = static_cast<xmlNodePtr>(start.get_raw_node());
-    if ((n = find_element(name, n)) != NULL)
+    auto n = static_cast<xmlNodePtr>(start.get_raw_node());
+    if ((n = find_element(name, n)) != nullptr)
         return iterator(n);
     return end();
 }
@@ -711,8 +684,8 @@ node::iterator node::find(const char *name, const iterator& start)
 
 node::const_iterator node::find(const char *name, const const_iterator& start) const
 {
-    xmlNodePtr n = static_cast<xmlNodePtr>(start.get_raw_node());
-    if ((n = find_element(name, n)) != NULL)
+    auto n = static_cast<xmlNodePtr>(start.get_raw_node());
+    if ((n = find_element(name, n)) != nullptr)
         return const_iterator(n);
     return end();
 }
@@ -756,7 +729,7 @@ xml::const_nodes_view node::elements(const char *name) const
 
 node::iterator node::insert(const node &n)
 {
-    return iterator(xml::impl::node_insert(pimpl_->xmlnode_, 0, n.pimpl_->xmlnode_));
+    return iterator(xml::impl::node_insert(pimpl_->xmlnode_, nullptr, n.pimpl_->xmlnode_));
 }
 
 
@@ -810,16 +783,16 @@ void node::clear()
 
     xmlFreeNodeList(n->children);
     n->children =
-    n->last = NULL;
+    n->last = nullptr;
 }
 
 
 void node::sort(const char *node_name, const char *attr_name)
 {
-    xmlNodePtr i(pimpl_->xmlnode_->children), next(0);
+    xmlNodePtr i(pimpl_->xmlnode_->children), next(nullptr);
     std::vector<xmlNodePtr> node_list;
 
-    while (i!=0)
+    while (i!=nullptr)
     {
         next = i->next;
 
@@ -842,10 +815,10 @@ void node::sort(const char *node_name, const char *attr_name)
 
 void node::sort_fo(cbfo_node_compare& cb)
 {
-    xmlNodePtr i(pimpl_->xmlnode_->children), next(0);
+    xmlNodePtr i(pimpl_->xmlnode_->children), next(nullptr);
     std::vector<xmlNodePtr> node_list;
 
-    while (i!=0)
+    while (i!=nullptr)
     {
         next = i->next;
 
