@@ -55,7 +55,7 @@ namespace impl
 class iter_advance_functor
 {
 public:
-    iter_advance_functor() : refcnt_(1) {}
+    iter_advance_functor() = default;
 
     void inc_ref()
     {
@@ -72,22 +72,22 @@ public:
 
 protected:
     // use inc_ref(), dec_ref() instead of using the dtor explicitly
-    virtual ~iter_advance_functor() {}
+    virtual ~iter_advance_functor() = default;
 
 private:
     // no copy ctor or assignment
-    iter_advance_functor(const iter_advance_functor& other);
-    iter_advance_functor& operator=(const iter_advance_functor& other);
+    iter_advance_functor(const iter_advance_functor& other) = delete;
+    iter_advance_functor& operator=(const iter_advance_functor& other) = delete;
 
 private:
-    int refcnt_;
+    int refcnt_{1};
 };
 
 // base iterator class
 class node_iterator
 {
 public:
-    node_iterator() : fake_node_(0), node_(0) {}
+    node_iterator() : fake_node_(0), node_(nullptr) {}
 
     node_iterator(node &parent)
         : fake_node_(0),
