@@ -88,17 +88,17 @@ public:
             m_next[table[i]] = table[i+1];
     }
 
-    virtual xmlNodePtr operator()(xmlNodePtr node) const
+    xmlNodePtr operator()(xmlNodePtr node) const override
     {
         const NextNodeMap::const_iterator i = m_next.find(node);
         if ( i == m_next.end() )
-            return NULL;
+            return nullptr;
         else
             return i->second;
     }
 
 private:
-    typedef std::map<xmlNodePtr, xmlNodePtr> NextNodeMap;
+    using NextNodeMap = std::map<xmlNodePtr, xmlNodePtr>;
     NextNodeMap m_next;
 };
 
@@ -126,7 +126,7 @@ struct xpath_context_impl
     template<typename NodesView>
     NodesView evaluate(const std::string& expr, node& n, error_handler& on_error)
     {
-        xmlNodePtr xmlnode = reinterpret_cast<xmlNodePtr>(n.get_node_data());
+        auto xmlnode = reinterpret_cast<xmlNodePtr>(n.get_node_data());
         if ( xmlnode->doc != ctxt_->doc )
         {
             throw xml::exception("node doesn't belong to context's document");
@@ -154,22 +154,19 @@ struct xpath_context_impl
 
 private:
     // non-copyable
-    xpath_context_impl(const xpath_context_impl&);
-    xpath_context_impl& operator=(const xpath_context_impl&);
+    xpath_context_impl(const xpath_context_impl&) = delete;
+    xpath_context_impl& operator=(const xpath_context_impl&) = delete;
 };
 
 } // namespace impl
 
 
 xpath_context::xpath_context(const document& doc)
+    : pimpl_{new impl::xpath_context_impl(doc)}
 {
-    pimpl_ = new impl::xpath_context_impl(doc);
 }
 
-xpath_context::~xpath_context()
-{
-    delete pimpl_;
-}
+xpath_context::~xpath_context() = default;
 
 void xpath_context::register_namespace(const std::string& prefix, const std::string& href)
 {
